from django.contrib.auth.models import Group
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned

from codelist.models import KonovaCode
from codelist.settings import CODELIST_CONSERVATION_OFFICE_ID, CODELIST_REGISTRATION_OFFICE_ID
from konova.management.commands.kspMigrater.base_migrater import BaseMigrater
from konova.settings import ZB_GROUP, ETS_GROUP
from user.models import User


class UserMigrater(BaseMigrater):

    def migrate(self):
        self.connect_db(db_name="natportal")
        cursor = self.db_connection.cursor()

        el = "'%§§15§§%'"
        cursor.execute(
            'select '
                'kennung, '
                'name, '
                'vorname, '
                'email, '
                'anwendungsnutzer, '
                'firmenname '
            'from berechtigung.nutzer where '
                'aktiv=true and '
                f'anwendungsnutzer like {el}'
        )
        fetch_results = cursor.fetchall()

        len_all_users = len(fetch_results)
        print(f"Migrate users...")
        print(f"--Found {len_all_users} entries. Process now...")
        num_processed = 0
        for result in fetch_results:
            if num_processed % 500 == 0:
                print(f"----{num_processed}/{len_all_users} processed")
            user = User.objects.get_or_create(
                username=result[0]
            )[0]
            user.last_name = result[1]
            user.first_name = result[2]
            user.email = result[3]
            user.save()

            appusers = result[4].split(";;;")
            user_teams = []
            company_team = None
            for tmp in appusers:
                if "§§15§§" not in tmp:
                    continue
                contents = tmp.split("§§")
                _type = contents[2]
                _org = contents[3]
                ets_id = "20"
                zb_id = "26"
                is_type_ets = _type == ets_id
                is_type_zb = _type == zb_id
                if not _type or not _org:
                    # broken - unusable
                    continue

                try:
                    if is_type_zb:
                        _org = KonovaCode.objects.get(
                            atom_id=_org,
                            code_lists__in=[CODELIST_REGISTRATION_OFFICE_ID]
                        )
                        team = self._get_team_from_responsible_code(responsible_code=_org, prefix="ZB")
                        user_teams.append(team)
                        self.give_registration_office_permission_for_user(user)
                    elif is_type_ets:
                        _org = KonovaCode.objects.get(
                            atom_id=_org,
                            code_lists__in=[CODELIST_CONSERVATION_OFFICE_ID]
                        )
                        team = self._get_team_from_responsible_code(responsible_code=_org, prefix="ETS")
                        user_teams.append(team)
                        self.give_conservation_office_permission_for_user(user)
                    else:
                        _org = result[5]
                        if company_team is None:
                            company_team = self._get_team(_org)
                except ObjectDoesNotExist:
                    # organisation code might not be valid anymore
                    continue
                except MultipleObjectsReturned as e:
                    raise MultipleObjectsReturned(f"{_org}: {e}")
            if company_team is not None and len(user_teams) == 0:
                # Only team is the company team
                company_team.users.add(user)
                company_team.admins.add(user)
            else:
                if company_team is not None:
                    company_team.delete()
                for team in user_teams:
                    team.users.add(user)
                    team.admins.add(user)
            num_processed += 1
        cursor.close()

    def give_registration_office_permission_for_user(self, user: User):
        """ Gives user registration office group permissions

        """
        group = Group.objects.get(
            name=ZB_GROUP,
        )
        user.groups.add(group)
        return user

    def give_conservation_office_permission_for_user(self, user: User):
        """ Gives user conservation office group permissions

        """
        group = Group.objects.get(
            name=ETS_GROUP,
        )
        user.groups.add(group)
        return user
